-module(draw).

-export( [ main/0, export/1, emptyDrawing/2, add/2, deleteDrawing/1, width/1, height/1, rect/4, rect/5, circle/3, circle/4, line/6, line/4, path/2, pathLineTo/3, pathMoveTo/3 ] ).


%----------------------------------------------
emptyDrawing(Width, Height) ->
    { { canevas, Width, Height } , [ ] }.

add(Figure, Drawing) ->
    { Canevas, Figures } = Drawing,
    { Canevas, [ Figure | Figures ] }.

deleteDrawing(Drawing) ->
    { { canevas, Width, Height } , _ } = Drawing,
    { { canevas, Width, Height } , [ ] }.
    
width({ { canevas, Width, _ } , _ }) ->
    Width.

height({ { canevas, _, Height } , _ }) ->
    Height.
    
rect(X, Y, Width, Height, Color) ->
    { rect, X, Y, Width, Height, Color }.

rect(X, Y, Width, Height) ->
    { rect, X, Y, Width, Height, "black" }.

circle(CX, CY, R, Color) ->
    { circle, CX, CY, R, Color }.

circle(CX, CY, R) ->
    { circle, CX, CY, R, "black" }.

line(X1, Y1, X2, Y2, Color, W) ->
    { line, X1, Y1, X2, Y2, Color, W }.

line(X1, Y1, X2, Y2) ->
    { line, X1, Y1, X2, Y2, "black", 2 }.

path(X, Y) ->
    { path, [ { moveTo, X, Y } ] }.

pathLineTo(X, Y, { path, Points } ) ->
    { path, [ { lineTo, X, Y } | Points ] }.

pathMoveTo(X, Y, { path, Points } ) ->
    { path, [ { moveTo, X, Y } | Points ] }.

%----------------------------------------------
export(Drawing) ->
    {ok, Device} = file:open("drawing.svg", [write]),
    { { canevas, Width, Height } , Figures } = Drawing,
    headFile(Device, Width, Height),
    bodyFile(Device, Figures),
    bottomFile(Device),
    file:close(Device).

headFile(Device, Width, Height) ->
    io:fwrite(Device, "<?xml version=\"1.0\" encoding=\"utf-8\"?>~n", []),
    io:fwrite(Device, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\" ~n", []),
    io:fwrite(Device, "  \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">~n", []),
    io:fwrite(Device, "<svg width=\"~B\" height=\"~B\" ~n", [Width, Height]),
    io:fwrite(Device, "     xml:lang=\"fr\" xmlns=\"http://www.w3.org/2000/svg\" ~n", []),
    io:fwrite(Device, "     xmlns:xlink=\"http://www.w3.org/1999/xlink\">~n", []),
    io:fwrite(Device, "<title>Ecran</title>~n", []).

bottomFile(Device) ->
    io:fwrite(Device, "</svg>~n", []).

bodyFile(_, []) ->
    ok;
bodyFile(Device, [ Head | Tail ]) ->
    bodyFile(Device, Tail),
    case Head of
	{ rect, X, Y, W, H, Color } -> 
	    io:fwrite(Device, "<rect x=\"~B\" y=\"~B\" width=\"~B\" height=\"~B\" fill=\"~s\"/>~n", [X, Y, W, H, Color]);
	{ circle, CX, CY, R, Color } -> 
	    io:fwrite(Device, "<circle cx=\"~B\" cy=\"~B\" r=\"~B\" fill=\"~s\"/>~n", [CX, CY, R, Color]);
	{ line, X1, Y1, X2, Y2, Color, Width } -> 
	    io:fwrite(Device, "<line x1=\"~B\" y1=\"~B\" x2=\"~B\" y2=\"~B\" stroke=\"~s\" stroke-width=\"~B\" />~n", [X1, Y1, X2, Y2, Color, Width]);
	{ path, Points } -> 
	    io:fwrite(Device, "<path d=\"~s\" style=\"fill:none; stroke:black\"/>~n", [ dataPath(Points) ])
    end.

dataPath([ ]) ->
    "";
dataPath( [ Head | Tail ] ) ->
    case Head of
	{ moveTo, X, Y } ->
	    dataPath(Tail) ++ " M " ++ integer_to_list(X) ++ "," ++ integer_to_list(Y) ;
	{ lineTo, X, Y } ->
	    dataPath(Tail) ++ " L " ++ integer_to_list(X) ++ "," ++ integer_to_list(Y) 
    end.

	
%----------------------------------------------
% Main function
%----------------------------------------------
main() ->
    P = draw:pathLineTo(250, 100,
			draw:pathLineTo(400, 300,
					draw:path(20, 20))),

    D = draw:add(
    	  P,
    	  draw:add(
    	    draw:line(40, 100, 0, 10, "blue", 4), 
    	    draw:add(draw:circle(100, 50, 40, "red"), 
    		     draw:add(draw:rect(20, 40, 50, 50), 
    			      draw:emptyDrawing(400, 400))))),

    draw:export(D).


    
